class NodeClassesController < InheritedResources::Base
  respond_to :html, :json
  custom_actions :collection => [:select]
  custom_actions :resource => [:select_parameters]
  actions :edit, :index, :show, :create, :update, :destroy, :select, :select_parameters 
  before_filter :raise_unless_using_external_node_classification
  before_filter :raise_if_enable_read_only_mode, :only => [:edit, :create, :update, :destroy, :select, :select_parameters]

  include SearchableIndex
  include ActionView::Helpers::TextHelper

  def select(options={}, &block)
    respond_with(options, &block)
  end

  def docs(options={}, &block)
    docs = ClassDiscoveryHelper.get_class_details(params[:name])["doc"]
    h = RDoc::Markup::ToHtml.new
    render :json => { :short => ClassDiscoveryHelper.labelize_description(params[:name], docs), :full => h.convert(docs) }, :content_type => 'application/json'
  end 
  
  #This is an auxiliary class that acts as an ActiveRecord object when queried for errors.
  #It's used for collecting errors from multiple active record objects being processed in a batch transaction.
  class ErrorHelper
    attr_reader :errors
    
    def initialize
      @errors = ActiveModel::Errors.new(self)
    end
  end
  
  def create
    if params[:node_classes].nil?
      super
    else
      error_helper = ErrorHelper.new
      
      classes = JSON.parse(params[:node_classes])

      if classes.empty?
        
        error_helper.errors.add(:base, "Please, select at least one class to add") 

        html = render_to_string(:template => "shared/_error",
                                :layout => false,
                                :locals => { :object => error_helper })
        render :json => { :status => "error", :error_html => html }, :content_type => 'application/json'

        return
      end

      begin
        discovered_class_docs = {}
        class_discovery_success = false
        begin
          ClassDiscoveryHelper.get_available_classes('*').each do |c|
            discovered_class_docs[c['name']] = c['doc']
          end
          class_discovery_success = true
        rescue => e
        end

        ActiveRecord::Base.transaction do
          
          class_discovery_problem = false

          classes.each do |node_class_name, node_class_desc|

            #if a discovered class is being added (which has a null description unlike manually added
            #which has at least an empty string) then a discovery failure should cancel the transaction            
            if node_class_desc.nil? && !class_discovery_success then
              class_discovery_problem = true
            end

            node_class = NodeClass.new
            node_class.name = node_class_name
            node_class.description = discovered_class_docs[node_class_name] || node_class_desc
            node_class.save

            node_class.errors.full_messages.each do |error|
              error_helper.errors.add(:base, "class '#{node_class_name}': #{error}")
            end
          end

          if class_discovery_problem then
            error_helper.errors.add(:base, "could not retrieve class documentation from your master")
          end

          if !error_helper.errors.empty?
            raise ActiveRecord::Rollback
          end
        end
      rescue => e
        error_helper.errors.add(:base, e.message)
      end
      
      if error_helper.errors.empty?
        flash[:notice] = "#{pluralize(classes.count, 'class')} #{classes.count == 1 ? 'was' : 'were'} successfully created." 
        render :json => { :status => "ok", :valid => "true", :redirect_to => node_classes_path }, :content_type => 'application/json'
      else
        html = render_to_string(:template => "shared/_error",
                                :layout => false,
                                :locals => { :object => error_helper })
        render :json => { :status => "error", :error_html => html }, :content_type => 'application/json'
      end
    end
  end

  def select_parameters
    respond_with({}) do |success|
      clazz = NodeClass.find_by_id(params[:id])        
      class_details = ClassDiscoveryHelper.get_class_details(clazz.name)
      params = class_details ? class_details["parameters"] : {}
      if params.nil?
        params = {}
      end
      
      success.json { render :json => params, :status => 201 }
    end
  end

  protected

  def resource
    get_resource_ivar || set_resource_ivar(end_of_association_chain.find_by_id_or_name!(params[:id]))
  end
end
